import sys
import requests
import xbmc
import xbmcgui
import xbmcplugin
import xbmcaddon
import json
from urllib.parse import quote, urlencode, parse_qsl
from concurrent.futures import ThreadPoolExecutor, as_completed
# Funzioni importate
from login import check_credentials
import updater
from mpd_playback import play_mpd_content
from hls_playback import play_hls_content

# Informazioni sull'addon
addon = xbmcaddon.Addon()
addon_handle = int(sys.argv[1])
base_url = sys.argv[0]

def get_file_from_api(file_path):
    # Costruisci l'URL per la richiesta
    url = f'https://monflix.atwebpages.com/api_json.php?path={quote(file_path)}'
    
    try:
        # Effettua la richiesta GET
        response = requests.get(url)
        response.raise_for_status()  # Controlla se ci sono errori HTTP

        # Log della risposta ricevuta
        xbmc.log(f"Risposta ricevuta da {url}: {response.text[:100]}", xbmc.LOGDEBUG)

        # Determina il tipo di contenuto dalla risposta e dall'estensione del file
        content_type = response.headers.get('Content-Type', '').lower()
        file_extension = file_path.lower().split('.')[-1]

        # Se il file è .m3u o .m3u8, restituisci il contenuto come testo
        if file_extension in ['m3u', 'm3u8']:
            return response.text
        
        # Per i file JSON, prova a decodificare
        if 'application/json' in content_type or file_extension == 'json':
            try:
                return response.json()
            except json.JSONDecodeError:
                # Se la decodifica JSON fallisce, restituisci il testo grezzo
                return response.text
        elif 'text/plain' in content_type or 'application/octet-stream' in content_type:
            # Se è testo o file generico, restituisci il contenuto come stringa
            return response.text
        else:
            # Tipo di contenuto sconosciuto
            xbmc.log(f"Tipo di contenuto sconosciuto ricevuto: {content_type}", xbmc.LOGERROR)
            return None
            
    except requests.exceptions.HTTPError as e:
        if e.response.status_code == 404:
            xbmc.log(f"File non trovato: {file_path}. URL: {url}", xbmc.LOGERROR)
        else:
            xbmc.log(f"Errore HTTP nel recupero del file {file_path}: {e}", xbmc.LOGERROR)
    except requests.exceptions.RequestException as e:
        xbmc.log(f"Errore nel recupero del file {file_path}: {str(e)}", xbmc.LOGERROR)
    except ValueError as e:
        xbmc.log(f"Errore nella decodifica del contenuto ricevuto da {file_path}: {str(e)}", xbmc.LOGERROR)

    return None

# Funzione per costruire un URL con parametri
def build_url(query):
    return f"{base_url}?{urlencode(query)}"

def save_credentials(username, password):
    # Salva le credenziali nel file settings.xml
    addon.setSetting('username', username)
    addon.setSetting('password', password)

def login():
    dialog = xbmcgui.Dialog()

    # Controlla se ci sono credenziali salvate
    saved_username = addon.getSetting('username')
    saved_password = addon.getSetting('password')

    if saved_username and saved_password:
        # Se esistono, usa le credenziali salvate
        if check_credentials(saved_username, saved_password):
            dialog.notification('Login', 'Login effettuato con successo', xbmcgui.NOTIFICATION_INFO, 3000)
            return True
        else:
            dialog.notification('Login', 'Credenziali scadute o non valide, reinserisci.', xbmcgui.NOTIFICATION_ERROR, 3000)

    # Se non ci sono credenziali salvate, chiedi all'utente di inserirle
    username = dialog.input('Inserisci Username')
    password = dialog.input('Inserisci Password', option=xbmcgui.ALPHANUM_HIDE_INPUT)
    
    if check_credentials(username, password):
        dialog.notification('Login', 'Login effettuato con successo', xbmcgui.NOTIFICATION_INFO, 3000)
        
        # Salva le credenziali nel file settings.xml
        save_credentials(username, password)

        return True
    else:
        dialog.notification('Login', 'Login fallito', xbmcgui.NOTIFICATION_ERROR, 3000)
        return False

# Funzione per mostrare il menu principale
def main_menu():
    if not login():
        return
    
    updater.check_for_updates()
    # Categorie principali
    categories = [
        {"name": "Live TV", "icon": "https://repomon.atwebpages.com/loghi/monflixtv.jpg", "poster": "https://repomon.atwebpages.com/fanart/livetv.webp"},
        {"name": "Film", "icon": "https://repomon.atwebpages.com/loghi/film.jpg", "poster": "https://repomon.atwebpages.com/fanart/film2.jpg"},
        {"name": "Serie TV", "icon": "https://repomon.atwebpages.com/loghi/serie.jpg", "poster": "https://repomon.atwebpages.com/fanart/serie2.jpg"}
        ]
    for category in categories:
        list_item = xbmcgui.ListItem(label=category["name"])
        list_item.setArt({"icon": category["icon"], "fanart": category["poster"]})
        url = build_url({'action': 'open_category', 'category': category["name"]})
        xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=True)

    xbmcplugin.endOfDirectory(addon_handle)

# Funzione per aprire una categoria
def open_category(category):
    try:
        if category == "Film":
            search_item = xbmcgui.ListItem(label="Cerca Film")
            search_item.setArt({
                "icon": "https://external-content.duckduckgo.com/iu/?u=https%3A%2F%2Fthumbs.dreamstime.com%2Fb%2Flente-di-ingrandimento-o-icona-e-concetto-ricerca-isolato-sul-nero-illustrazione-vettoriale-182024092.jpg&f=1&nofb=1&ipt=4c01aa99e5e615ed3b1cc949b216a5363f456e2742fcb59fca2bf41cdc36fb58&ipo=images",
                "fanart": "https://external-content.duckduckgo.com/iu/?u=https%3A%2F%2Fstatic.vecteezy.com%2Fsystem%2Fresources%2Fpreviews%2F000%2F660%2F143%2Flarge_2x%2Fvector-movie-theater-icons.jpg&f=1&nofb=1&ipt=212a74272f7b1d18cc45a09b224b91ed3434c0919de8114fb08c145ed8abb92b&ipo=images"
            })
            search_url = build_url({'action': 'search_movies'})
            xbmcplugin.addDirectoryItem(handle=addon_handle, url=search_url, listitem=search_item, isFolder=True)
            subcategories = [
                {"name": "Animazione", "icon": "https://repomon.atwebpages.com/icon/A.png", "poster": "https://repomon.atwebpages.com/fanart/animazione.jpg"},
                {"name": "Azione", "icon": "https://repomon.atwebpages.com/icon/A.png", "poster": "https://repomon.atwebpages.com/fanart/azione.jpg"},
                {"name": "Avventura", "icon": "https://repomon.atwebpages.com/icon/A.png", "poster": "https://repomon.atwebpages.com/fanart/avventura.jpg"},
                {"name": "Commedia", "icon": "https://repomon.atwebpages.com/icon/C.png", "poster": "https://repomon.atwebpages.com/fanart/commedia.webp"},
                {"name": "Fantascienza", "icon": "https://repomon.atwebpages.com/icon/F.png", "poster": "https://repomon.atwebpages.com/fanart/fantascienza.jpg"},
                {"name": "Guerra", "icon": "https://repomon.atwebpages.com/icon/G.png", "poster": "https://repomon.atwebpages.com/fanart/guerra.jpg"},
                {"name": "Horror", "icon": "https://repomon.atwebpages.com/icon/H.png", "poster": "https://repomon.atwebpages.com/fanart/horror.webp"},
                {"name": "Dramma", "icon": "https://repomon.atwebpages.com/icon/D.png", "poster": "https://repomon.atwebpages.com/fanart/dramma.webp"},
                {"name": "Family", "icon": "https://repomon.atwebpages.com/icon/F.png", "poster": "https://repomon.atwebpages.com/fanart/family.jpg"},
                {"name": "Crime", "icon": "https://repomon.atwebpages.com/icon/C.png", "poster": "https://repomon.atwebpages.com/fanart/crime.jpg"},
                {"name": "Story", "icon": "https://repomon.atwebpages.com/icon/S.png", "poster": "https://repomon.atwebpages.com/fanart/story.jpg"},
                {"name": "Mistery", "icon": "https://repomon.atwebpages.com/icon/M.png", "poster": "https://repomon.atwebpages.com/fanart/mistery.jpg"},
                {"name": "Romance", "icon": "https://repomon.atwebpages.com/icon/R.png", "poster": "https://repomon.atwebpages.com/fanart/romance.jpg"},
                {"name": "Thriller", "icon": "https://repomon.atwebpages.com/icon/T.png", "poster": "https://repomon.atwebpages.com/fanart/thriller.webp"},
                {"name": "Western", "icon": "https://repomon.atwebpages.com/icon/W.jpg", "poster": "https://repomon.atwebpages.com/fanart/western.jpg"}
                ]
            for subcategory in subcategories:
                list_item = xbmcgui.ListItem(label=subcategory["name"])
                list_item.setArt({"icon": subcategory["icon"], "fanart": subcategory["poster"]})
                url = build_url({'action': 'initCategory', 'subcategory': subcategory["name"]})
                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=True)

        elif category == "Serie TV":
            load_series()

        elif category == "Live TV":
            live_tv_subcategories = [
            {"name": "SkySport", "icon": "https://repomon.atwebpages.com/loghi/skysport.jpg", "poster": "https://repomon.atwebpages.com/fanart/skysport.png"},
            {"name": "Dazn", "icon": "https://repomon.atwebpages.com/loghi/dazn.jpg", "poster": "https://repomon.atwebpages.com/fanart/dazn.jpg"},
            {"name": "Sport", "icon": "https://repomon.atwebpages.com/loghi/sport.jpg", "poster": "https://repomon.atwebpages.com/fanart/sport.jpg"},
            {"name": "Eventi in Vod", "icon": "https://repomon.atwebpages.com/loghi/vod.jpg", "poster": "https://repomon.atwebpages.com/fanart/vod.jpg"},
            {"name": "Intrattenimento", "icon": "https://repomon.atwebpages.com/loghi/sky-intra.jpg", "poster": "https://repomon.atwebpages.com/fanart/sky-intra.jpg"},
            {"name": "Cinema", "icon": "https://repomon.atwebpages.com/loghi/skycinema.jpg", "poster": "https://repomon.atwebpages.com/fanart/skycinema.webp"},
            {"name": "Kids", "icon": "https://repomon.atwebpages.com/loghi/sky-intra.jpg", "poster": "https://repomon.atwebpages.com/fanart/kids.jpg"},
            {"name": "TV DTT", "icon": "https://repomon.atwebpages.com/loghi/dtt.jpg", "poster": "https://repomon.atwebpages.com/fanart/dtt.jpg"}
        ]
            for subcategory in live_tv_subcategories:
                list_item = xbmcgui.ListItem(label=subcategory["name"])
                list_item.setArt({"icon": subcategory["icon"], "fanart": subcategory["poster"]})
                url = build_url({'action': 'loadLiveTV', 'subcategory': subcategory["name"]})
                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=True)

        xbmcplugin.endOfDirectory(addon_handle)

    except Exception as e:
        xbmcgui.Dialog().notification('Errore', f"Errore caricamento categoria: {str(e)}", xbmcgui.NOTIFICATION_ERROR, 5000)
        xbmc.log(f"Exception in open_category: {str(e)}", xbmc.LOGERROR)

def search_in_json_file(json_url, search_term):
    channels = load_json(json_url)
    if not channels:
        return []
        
    found = []
    for movie in channels:
        if not isinstance(movie, dict):
            continue
        title = movie.get('name', '').lower()
        if search_term in title:
            found.append(movie)
    return found

def search_movies():
    dialog = xbmcgui.Dialog()
    search_term = dialog.input('Cerca Film', type=xbmcgui.INPUT_ALPHANUM)
    
    if not search_term:
        return
    
    search_term = search_term.lower()
    
    try:
        # Ottieni tutte le URL dei file JSON
        all_json_urls = []
        categories = ["Animazione", "Azione", "Avventura", "Commedia", "Fantascienza", 
                     "Guerra", "Horror", "Dramma", "Family", "Crime", "Story", 
                     "Mistery", "Romance", "Thriller", "Western"]
        
        for category in categories:
            all_json_urls.extend(get_json_urls_for_category(category))
        
        found_movies = []
        # Usa ThreadPoolExecutor per cercare in parallelo
        with ThreadPoolExecutor(max_workers=10) as executor:
            future_to_url = {executor.submit(search_in_json_file, url, search_term): url 
                           for url in all_json_urls}
            
            # Progress dialog
            progress = xbmcgui.DialogProgress()
            progress.create('Ricerca in corso', 'Cercando nei film...')
            total_files = len(all_json_urls)
            completed = 0
            
            for future in as_completed(future_to_url):
                completed += 1
                progress.update(int((completed / total_files) * 100))
                
                found_movies.extend(future.result())
                
                if progress.iscanceled():
                    progress.close()
                    return
            
            progress.close()

        if not found_movies:
            dialog.notification('Ricerca', 'Nessun film trovato', xbmcgui.NOTIFICATION_INFO, 3000)
            return
            
        # Mostra i risultati
        for movie in found_movies:
            title = movie['name']
            info_film = get_info(title, "movie")
            
            if info_film and len(info_film["results"]) > 0:
                movie_info = info_film["results"][0]
                
                poster_path = movie_info.get('poster_path', '')
                backdrop_path = movie_info.get('backdrop_path', '')
                overview = movie_info.get('overview', 'Descrizione non disponibile')

                list_item = xbmcgui.ListItem(label=title)
                list_item.setInfo('video', {
                    'title': title,
                    'plot': overview
                })

                list_item.setArt({
                    'thumb': f"https://media.themoviedb.org/t/p/w300_and_h450_bestv2/{poster_path}" if poster_path else '',
                    'fanart': f"https://media.themoviedb.org/t/p/w300_and_h450_bestv2/{backdrop_path}" if backdrop_path else ''
                })
                
                url = build_url({
                    'action': 'play', 
                    'content_type': 'movie', 
                    'title': title, 
                    'hls_url': movie.get('hls_url', '')  # Prendi solo l'URL HLS
                })
                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=False)
        
        xbmcplugin.endOfDirectory(addon_handle)
        
    except Exception as e:
        xbmc.log(f"Errore durante la ricerca: {str(e)}", xbmc.LOGERROR)
        dialog.notification('Errore', 'Errore durante la ricerca', xbmcgui.NOTIFICATION_ERROR, 3000)
        
    except Exception as e:
        xbmc.log(f"Errore durante la ricerca: {str(e)}", xbmc.LOGERROR)
        dialog.notification('Errore', 'Errore durante la ricerca', xbmcgui.NOTIFICATION_ERROR, 3000)

def play_live_tv_channels(channels):
    try:
        xbmc.log(f"Numero di canali caricati: {len(channels)}", xbmc.LOGINFO)
        
        for channel in channels:
            if 'staff_id' in channel and 'name' in channel:
                list_item = xbmcgui.ListItem(label=channel['name'])
                list_item.setInfo('video', {'title': channel['name']})
                list_item.setProperty('IsPlayable', 'true')
                if 'logo' in channel:
                    list_item.setArt({'thumb': channel['logo'], 'icon': channel['logo'], 'fanart': channel['logo']})
                url = build_url({'action': 'play', 'content_type': 'live', 'id': channel['staff_id'], 'mpd_url': channel})
                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=False)

        xbmcplugin.endOfDirectory(addon_handle)
    
    except Exception as e:
        xbmc.log(f"Errore in play_live_tv_channels: {str(e)}", level=xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', 'Errore durante il caricamento dei Live', xbmcgui.NOTIFICATION_ERROR, 5000)

def play_film_channels():
    try:
        # Carica la lista dei film da un file o da una fonte di dati
        films = load_movie_list()  # Assicurati che questa funzione sia implementata correttamente
        if not films:
            xbmcgui.Dialog().notification('Errore', 'Nessun film trovato', xbmcgui.NOTIFICATION_ERROR, 3000)
            return

        for film in films:
            if 'staff_id' in film and 'name' in film and 'hls_url' in film:
                list_item = xbmcgui.ListItem(label=film['name'])
                list_item.setInfo('video', {'title': film['name']})
                list_item.setProperty('IsPlayable', 'true')
                if 'logo' in film:
                    list_item.setArt({'thumb': film['logo'], 'icon': film['logo'], 'fanart': film['logo']})

                # Codifica l'URL HLS per evitare problemi con caratteri speciali
                encoded_hls_url = quote(film['hls_url'])
                
                # Crea un dizionario con i parametri per build_url
                url_params = {
                    'action': 'play',
                    'content_type': 'film',
                    'id': film['staff_id'],
                    'hls_url': encoded_hls_url
                }
                
                url = build_url(url_params)
                
                xbmc.log(f"URL costruito per il film {film['name']}: {url}", level=xbmc.LOGDEBUG)

                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=False)

        xbmcplugin.endOfDirectory(addon_handle)
    except Exception as e:
        xbmc.log(f"Errore in play_film_channels: {str(e)}", level=xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', 'Errore durante il caricamento dei film', xbmcgui.NOTIFICATION_ERROR, 5000)

def play_episode(episode):
    try:
        xbmc.log(f"Avvio episodio: {episode['name']}", xbmc.LOGINFO)
        
        # Estrai le informazioni necessarie dall'episodio
        user_agent = 'User-Agent=Mozilla/5.0 (Linux; Android 11.0; Build/RTK2.230523.082) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36 CrKey/1.56.500000 DeviceType/AndroidTV'
        mpd_url = episode.get('mpd_url', '')
        license_key = episode.get('license_key', '')
        
        if not mpd_url:
            xbmc.log("Errore: mpd_url non trovato per l'episodio", xbmc.LOGERROR)
            xbmcgui.Dialog().notification('Errore', 'MPD URL non trovato', xbmcgui.NOTIFICATION_ERROR, 3000)
            return
        
        # Crea il ListItem con il path diretto all'MPD
        list_item = xbmcgui.ListItem(path=mpd_url)
        
        # Imposta le informazioni video
        list_item.setInfo('video', {
            'title': episode['name'],
            'plot': f"Staff ID: {episode.get('staff_id', 'N/A')}",
            'mediatype': 'episode'
        })
        
        # Configura inputstream adaptive per MPD/DASH
        list_item.setProperty('inputstream', 'inputstream.adaptive')
        list_item.setProperty('inputstream.adaptive.stream_headers', user_agent)
        list_item.setProperty('inputstream.adaptive.manifest_headers', user_agent)
        
        # Gestione delle chiavi ClearKey (singola o multiple)
        if license_key:
            if isinstance(license_key, str):
                # Caso singola coppia di chiavi
                list_item.setProperty('inputstream.adaptive.drm_legacy', f'org.w3.clearkey|{license_key}')
            elif isinstance(license_key, dict):
                # Caso multiple coppie di chiavi
                keys_string = ','.join([f"{kid}:{key}" for kid, key in license_key.items()])
                list_item.setProperty('inputstream.adaptive.drm_legacy', f'org.w3.clearkey|{keys_string}')
            else:
                xbmc.log("Formato chiave non riconosciuto", xbmc.LOGERROR)
                return
        
        # Se l'episodio ha una copertura visiva (logo, ecc.), imposta l'immagine
        if 'logo' in episode and episode['logo']:
            list_item.setArt({
                'thumb': episode['logo'],
                'fanart': episode['logo']
            })
        
        # Avvia la riproduzione direttamente con l'MPD URL
        xbmcplugin.setResolvedUrl(handle=addon_handle, succeeded=True, listitem=list_item)
        
    except Exception as e:
        xbmc.log(f"Errore durante il play dell'episodio: {str(e)}", xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', f'Errore durante il play dell\'episodio: {str(e)}', xbmcgui.NOTIFICATION_ERROR, 5000)

def get_info(title, content_type):
    tmdb_api_key = "ded5b7afc33c6be7e0eaefce452217b1"
    r = requests.get(f"https://api.themoviedb.org/3/search/{content_type}?api_key={tmdb_api_key}&language=it-IT&query={requests.utils.quote(title)}")
    if r  and r.json():
        return r.json()

def load_m3u(url):
    channels = []  # Inizializza la lista dei canali
    current_channel = {}  # Inizializza il dizionario per il canale corrente
    
    content = get_file_from_api(url)
    
    if content is None:
        xbmc.log(f"Impossibile caricare il file da Host: {url}", xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', f'Impossibile caricare il file: {url}', xbmcgui.NOTIFICATION_ERROR, 5000)
        return []
    
    # Divide il contenuto in righe e rimuove spazi extra
    lines = [line.strip() for line in content.splitlines() if line.strip()]
    
    # Verifica se è un file M3U valido
    if not lines or not lines[0].startswith('#EXTM3U'):
        xbmc.log("File M3U non valido", xbmc.LOGERROR)
        return []

    for line in lines:
        if line.startswith('#EXTINF'):
            current_channel = {}  # Reset per nuovo canale
            
            # Estrai le informazioni dal tag EXTINF
            try:
                # Divide la linea in due parti: attributi e nome
                info_part = line[line.find(',')+1:].strip()
                attrs_part = line[:line.find(',')]
                
                # Imposta il nome del canale
                current_channel['name'] = info_part
                
                # Estrai gli attributi (tvg-id, tvg-logo, ecc.)
                for attr in attrs_part.split():
                    if '=' in attr:
                        key, value = attr.split('=', 1)
                        value = value.strip('"\'')
                        if key == 'tvg-logo':
                            current_channel['logo'] = value
                        elif key == 'group-title':
                            current_channel['group_title'] = value
                        elif key == 'staff-id':
                            current_channel['staff_id'] = value
            except Exception as e:
                xbmc.log(f"Errore nel parsing della linea EXTINF: {str(e)}", xbmc.LOGERROR)
                continue
        elif line.startswith('#KODIPROP:'):
            try:
                prop_line = line.split(':', 1)[1]
                if 'license_key=' in prop_line:
                    current_channel['license_key'] = prop_line.split('=', 1)[1].strip()
                elif 'stream_headers=' in prop_line:
                    current_channel['stream_headers'] = prop_line.split('=', 1)[1].strip()
                elif 'drm_legacy=' in prop_line:
                    current_channel['drm_legacy'] = prop_line.split('=', 1)[1].strip()
            except Exception as e:
                xbmc.log(f"Errore nel parsing della linea KODIPROP: {str(e)}", xbmc.LOGERROR)
        elif line.startswith('http'):
            if current_channel:  # Verifica che ci sia un canale corrente
                current_channel['mpd_url'] = line
                channels.append(current_channel.copy())  # Aggiungi una copia del canale
    
    xbmc.log(f"Canali caricati: {len(channels)}", xbmc.LOGINFO)
    return channels

def load_json(url):
    content = get_file_from_api(url)
    if content is None:
        xbmc.log(f"Impossibile caricare il file JSON da repository: {url}", xbmc.LOGERROR)
        return []

    # Se content è già una lista o un dizionario, restituiscilo direttamente
    if isinstance(content, (list, dict)):
        return content

    try:
        if isinstance(content, str):
            # Rimuovi eventuali BOM solo se content è una stringa
            content = content.lstrip('\ufeff')
            # Prova a decodificare il JSON
            data = json.loads(content)
            return data
        else:
            xbmc.log(f"Tipo di contenuto non valido: {type(content)}", xbmc.LOGERROR)
            return []
    except json.JSONDecodeError as e:
        xbmc.log(f"Errore nella decodifica del file JSON: {url}. Errore: {str(e)}", xbmc.LOGERROR)
        xbmc.log(f"Contenuto problematico: {str(content)[:200]}...", xbmc.LOGDEBUG)
        return []

def loadLiveTV(subcategory):
    m3u_urls = []
    if subcategory == "SkySport":
        m3u_urls = [
            'json/tv/sport/cielo.m3u'
        ]
    elif subcategory == "Dazn":
        m3u_urls = [
            'json/tv/sport/dazn.m3u'
        ]
    elif subcategory == "Sport":
        m3u_urls = [
            'json/tv/sport/dask.m3u'
        ]
    elif subcategory == "Eventi in Vod":
        m3u_urls = [
            'json/tv/vod/vod.m3u'
        ]
    elif subcategory == "Intrattenimento":
        m3u_urls = [
            'json/tv/intra/intra.m3u'
        ]
    elif subcategory == "Cinema":
        m3u_urls = [
            'json/tv/cinema/cinema.m3u'
        ]
    elif subcategory == "Kids":
        m3u_urls = [
            'json/tv/intra/kids.m3u'
        ]
    elif subcategory == "TV DTT":
        m3u_urls = [
            'json/tv/dtt/dtt.m3u'
        ]

    # Log del numero di file JSON da caricare
    xbmc.log(f"Caricamento di {len(m3u_urls)} liste per la sottocategoria {subcategory}.", xbmc.LOGINFO)

    for m3u_url in m3u_urls:
        xbmc.log(f"Tentativo di caricamento M3U da: {m3u_url}", xbmc.LOGINFO)
        channels = load_m3u(m3u_url)
        if channels:
            xbmc.log(f"Canali caricati: {len(channels)}", xbmc.LOGINFO)
            play_live_tv_channels(channels)
        else:
            xbmc.log(f"Nessun canale caricato da {m3u_url}", xbmc.LOGERROR)

def initCategory(subcategory):
    # Ottieni tutte le URL JSON per la sottocategoria
    json_urls = get_json_urls_for_category(subcategory)  # Questa funzione contiene tutte le liste di URL come prima
    
    # Carica solo la prima lista e passa l'indice della lista corrente e il totale delle liste
    if json_urls:
        load_movie_list(json_urls[0], 0, len(json_urls), subcategory)

def load_movie_list(json_url, current_list_index, total_lists, subcategory):
    """
    Carica una singola lista di film e aggiunge un pulsante per la lista successiva se disponibile
    """
    try:
        channels = load_json(json_url)
        if not channels:
            xbmc.log(f"Nessun film trovato nel file {json_url}", xbmc.LOGERROR)
            return

        # Mostra i film della lista corrente
        for movie in channels:
            if not isinstance(movie, dict):
                continue

            title = movie.get('name', 'Titolo non disponibile')
            info_film = get_info(title, "movie")
            
            if info_film and len(info_film["results"]) > 0:
                movie_info = info_film["results"][0]
                
                poster_path = movie_info.get('poster_path', '')
                backdrop_path = movie_info.get('backdrop_path', '')
                overview = movie_info.get('overview', 'Descrizione non disponibile')

                list_item = xbmcgui.ListItem(label=title)
                list_item.setInfo('video', {
                    'title': title,
                    'plot': overview
                })

                list_item.setArt({
                    'thumb': f"https://media.themoviedb.org/t/p/w300_and_h450_bestv2/{poster_path}" if poster_path else '',
                    'fanart': f"https://media.themoviedb.org/t/p/w300_and_h450_bestv2/{backdrop_path}" if backdrop_path else ''
                })
                
                url = build_url({'action': 'play', 'content_type': 'movie', 'title': title, 'hls_url': movie})
                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=False)

        # Se ci sono altre liste disponibili, aggiungi il pulsante "Prossima Lista"
        if current_list_index < total_lists - 1:
            next_list_item = xbmcgui.ListItem(label=f'Prossima Lista ({current_list_index + 2}/{total_lists})')
            next_list_url = build_url({
                'action': 'load_next_list',
                'current_index': current_list_index + 1,
                'total_lists': total_lists,
                'subcategory': subcategory
            })
            xbmcplugin.addDirectoryItem(handle=addon_handle, url=next_list_url, listitem=next_list_item, isFolder=True)

        xbmcplugin.endOfDirectory(addon_handle)

    except Exception as e:
        xbmc.log(f"Errore nel caricamento della lista film: {str(e)}", xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', 'Errore nel caricamento della lista', xbmcgui.NOTIFICATION_ERROR, 3000)

def get_json_urls_for_category(subcategory):
    json_urls = []
    if subcategory == "Animazione":
        json_urls = [
            'json/film/Animazione/Animazione_part_1.json',
            'json/film/Animazione/Animazione_part_2.json',
            'json/film/Animazione/Animazione_part_3.json',
            'json/film/Animazione/Animazione_part_4.json',
            'json/film/Animazione/Animazione_part_5.json',
            'json/film/Animazione/Animazione_part_6.json'
        ]
    elif subcategory == "Azione":
        json_urls = [
            'json/film/Azione/Azione_part_1.json',
            'json/film/Azione/Azione_part_2.json',
            'json/film/Azione/Azione_part_3.json',
            'json/film/Azione/Azione_part_4.json',
            'json/film/Azione/Azione_part_5.json',
            'json/film/Azione/Azione_part_6.json',
            'json/film/Azione/Azione_part_7.json',
            'json/film/Azione/Azione_part_8.json',
            'json/film/Azione/Azione_part_9.json',
            'json/film/Azione/Azione_part_10.json',
            'json/film/Azione/Azione_part_11.json',
            'json/film/Azione/Azione_part_12.json',
            'json/film/Azione/Azione_part_13.json'
        ]
    elif subcategory == "Avventura":
        json_urls = [
            'json/film/Avventura/Avventura_part_1.json',
            'json/film/Avventura/Avventura_part_2.json',
            'json/film/Avventura/Avventura_part_3.json',
            'json/film/Avventura/Avventura_part_4.json',
            'json/film/Avventura/Avventura_part_5.json',
            'json/film/Avventura/Avventura_part_6.json',
            'json/film/Avventura/Avventura_part_7.json',
            'json/film/Avventura/Avventura_part_8.json',
            'json/film/Avventura/Avventura_part_9.json',
            'json/film/Avventura/Avventura_part_10.json',
            'json/film/Avventura/Avventura_part_11.json',
            'json/film/Avventura/Avventura_part_12.json',
            'json/film/Avventura/Avventura_part_13.json'
        ]
    elif subcategory == "Commedia":
        json_urls = [
            'json/film/Commedia/Commedia_part_1.json',
            'json/film/Commedia/Commedia_part_2.json',
            'json/film/Commedia/Commedia_part_3.json',
            'json/film/Commedia/Commedia_part_4.json',
            'json/film/Commedia/Commedia_part_5.json',
            'json/film/Commedia/Commedia_part_6.json',
            'json/film/Commedia/Commedia_part_7.json',
            'json/film/Commedia/Commedia_part_8.json',
            'json/film/Commedia/Commedia_part_9.json',
            'json/film/Commedia/Commedia_part_10.json',
            'json/film/Commedia/Commedia_part_11.json',
            'json/film/Commedia/Commedia_part_12.json',
            'json/film/Commedia/Commedia_part_13.json'
        ]
    elif subcategory == "Fantascienza":
        json_urls = [
            'json/film/Fantascienza/Fantascienza_part_1.json',
            'json/film/Fantascienza/Fantascienza_part_2.json',
            'json/film/Fantascienza/Fantascienza_part_3.json',
            'json/film/Fantascienza/Fantascienza_part_4.json',
            'json/film/Fantascienza/Fantascienza_part_5.json',
            'json/film/Fantascienza/Fantascienza_part_6.json',
            'json/film/Fantascienza/Fantascienza_part_7.json',
            'json/film/Fantascienza/Fantascienza_part_8.json',
            'json/film/Fantascienza/Fantascienza_part_9.json',
            'json/film/Fantascienza/Fantascienza_part_10.json'
        ]
    elif subcategory == "Guerra":
        json_urls = [
            'json/film/Guerra/Guerra_part_1.json',
            'json/film/Guerra/Guerra_part_2.json'
        ]
    elif subcategory == "Horror":
        json_urls = [
            'json/film/Horror/Horror_part_1.json',
            'json/film/Horror/Horror_part_2.json',
            'json/film/Horror/Horror_part_3.json',
            'json/film/Horror/Horror_part_4.json',
            'json/film/Horror/Horror_part_5.json',
            'json/film/Horror/Horror_part_6.json',
            'json/film/Horror/Horror_part_7.json',
            'json/film/Horror/Horror_part_8.json',
            'json/film/Horror/Horror_part_9.json',
            'json/film/Horror/Horror_part_10.json'
        ]
    elif subcategory == "Dramma":
        json_urls = [
            'json/film/Dramma/Dramma_part_1.json',
            'json/film/Dramma/Dramma_part_2.json',
            'json/film/Dramma/Dramma_part_3.json',
            'json/film/Dramma/Dramma_part_4.json',
            'json/film/Dramma/Dramma_part_5.json',
            'json/film/Dramma/Dramma_part_6.json',
            'json/film/Dramma/Dramma_part_7.json',
            'json/film/Dramma/Dramma_part_8.json',
            'json/film/Dramma/Dramma_part_9.json',
            'json/film/Dramma/Dramma_part_10.json',
            'json/film/Dramma/Dramma_part_11.json',
            'json/film/Dramma/Dramma_part_12.json',
            'json/film/Dramma/Dramma_part_13.json'
        ]
    elif subcategory == "Family":
        json_urls = [
            'json/film/Family/Famiglia_part_1.json',
            'json/film/Family/Famiglia_part_2.json',
            'json/film/Family/Famiglia_part_3.json',
            'json/film/Family/Famiglia_part_4.json',
            'json/film/Family/Famiglia_part_5.json',
            'json/film/Family/Famiglia_part_6.json',
            'json/film/Family/Famiglia_part_7.json',
            'json/film/Family/Famiglia_part_8.json',
            'json/film/Family/Famiglia_part_9.json',
            'json/film/Family/Famiglia_part_10.json',
            'json/film/Family/Famiglia_part_11.json'
        ]
    elif subcategory == "Crime":
        json_urls = [
            'json/film/Crime/Crime_part_1.json',
            'json/film/Crime/Crime_part_2.json',
            'json/film/Crime/Crime_part_3.json',
            'json/film/Crime/Crime_part_4.json',
            'json/film/Crime/Crime_part_5.json',
            'json/film/Crime/Crime_part_6.json',
            'json/film/Crime/Crime_part_7.json',
            'json/film/Crime/Crime_part_8.json',
            'json/film/Crime/Crime_part_9.json',
            'json/film/Crime/Crime_part_10.json',
            'json/film/Crime/Crime_part_11.json'
        ]
    elif subcategory == "Story":
        json_urls = [
            'json/film/Story/Storia_part_1.json',
            'json/film/Story/Storia_part_2.json',
            'json/film/Story/Storia_part_3.json',
            'json/film/Story/Storia_part_4.json'
        ]
    elif subcategory == "Mistery":
        json_urls = [
            'json/film/Mistery/Mistero_part_1.json',
            'json/film/Mistery/Mistero_part_2.json',
            'json/film/Mistery/Mistero_part_3.json',
            'json/film/Mistery/Mistero_part_4.json',
            'json/film/Mistery/Mistero_part_5.json',
            'json/film/Mistery/Mistero_part_6.json'
        ]
    elif subcategory == "Romance":
        json_urls = [
            'json/film/Romance/Romance_part_1.json',
            'json/film/Romance/Romance_part_2.json',
            'json/film/Romance/Romance_part_3.json',
            'json/film/Romance/Romance_part_4.json',
            'json/film/Romance/Romance_part_5.json',
            'json/film/Romance/Romance_part_6.json',
            'json/film/Romance/Romance_part_7.json',
            'json/film/Romance/Romance_part_8.json',
            'json/film/Romance/Romance_part_9.json',
            'json/film/Romance/Romance_part_10.json',
            'json/film/Romance/Romance_part_11.json'
        ]
    elif subcategory == "Thriller":
        json_urls = [
            'json/film/Thriller/Thriller_part_1.json',
            'json/film/Thriller/Thriller_part_2.json',
            'json/film/Thriller/Thriller_part_3.json',
            'json/film/Thriller/Thriller_part_4.json',
            'json/film/Thriller/Thriller_part_5.json',
            'json/film/Thriller/Thriller_part_6.json',
            'json/film/Thriller/Thriller_part_7.json',
            'json/film/Thriller/Thriller_part_8.json',
            'json/film/Thriller/Thriller_part_9.json',
            'json/film/Thriller/Thriller_part_10.json',
            'json/film/Thriller/Thriller_part_11.json',
            'json/film/Thriller/Thriller_part_12.json',
            'json/film/Thriller/Thriller_part_13.json'
        ]
    elif subcategory == "Western":
        json_urls = [
            'json/film/Western/Westerm_part_1.json'
        ]
    return json_urls

def load_series():
    series_list = load_json("json/serie/file.json")
    
    # Verifica che series_list sia una lista
    if not series_list or not isinstance(series_list, list):
        xbmcgui.Dialog().notification('Errore', 'Formato serie non valido', xbmcgui.NOTIFICATION_ERROR, 3000)
        return

    # Itera su ogni serie nella lista
    for series in series_list:
        if not series or 'name' not in series:
            xbmc.log("Serie saltata: dati non validi", xbmc.LOGWARNING)
            continue

        # Ottieni le informazioni da TMDB
        info_series = get_info(series['name'], "tv")
        
        if info_series and len(info_series["results"]) > 0:
            series_info = info_series["results"][0]
            
            # Estrai le informazioni da TMDB
            poster_path = series_info.get('poster_path', '')
            backdrop_path = series_info.get('backdrop_path', '')
            overview = series_info.get('overview', 'Descrizione non disponibile')
            
            list_item = xbmcgui.ListItem(label=series['name'])
            list_item.setInfo('video', {
                'title': series['name'],
                'plot': overview,
                'mediatype': 'tvshow'
            })
            
            list_item.setArt({
                'thumb': f"https://media.themoviedb.org/t/p/w300_and_h450_bestv2/{poster_path}" if poster_path else '',
                'fanart': f"https://media.themoviedb.org/t/p/w1280{backdrop_path}" if backdrop_path else ''
            })
            
            # Crea URL per accedere alle stagioni
            url = build_url({
                'action': 'show_seasons',
                'series_name': series['name'],
                'series_data': json.dumps(series)
            })
            
            xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=True)

    xbmcplugin.endOfDirectory(addon_handle)

def show_seasons(series_name, series_data):
    try:
        series = json.loads(series_data)
        if 'Season' not in series:
            xbmcgui.Dialog().notification('Errore', 'Nessuna stagione trovata', xbmcgui.NOTIFICATION_ERROR, 3000)
            return
        
        # Ottieni info da TMDB per i dettagli delle stagioni
        info_series = get_info(series_name, "tv")
        
        # Gestisci più stagioni
        seasons = series.get('Season', [])
        
        if info_series and len(info_series["results"]) > 0:
            series_id = info_series["results"][0]["id"]
            
            # Itera su tutte le stagioni
            for season_index, season in enumerate(seasons, 1):
                season_info = get_season_info(series_id, season_index)
                
                list_item = xbmcgui.ListItem(label=f'Stagione {season_index}')
                
                if season_info:
                    poster_path = season_info.get('poster_path', '')
                    overview = season_info.get('overview', 'Informazioni non disponibili')
                    
                    list_item.setInfo('video', {
                        'title': f'Stagione {season_index}',
                        'plot': overview,
                        'mediatype': 'season'
                    })
                    
                    list_item.setArt({
                        'thumb': f"https://media.themoviedb.org/t/p/w300_and_h450_bestv2/{poster_path}" if poster_path else ''
                    })
                
                url = build_url({
                    'action': 'show_episodes',
                    'series_name': series_name,
                    'season_index': season_index - 1,
                    'season_data': json.dumps(season)
                })
                
                xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=True)
        
        xbmcplugin.endOfDirectory(addon_handle)
        
    except Exception as e:
        xbmc.log(f"Error in show_seasons: {str(e)}", xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', f'Errore: {str(e)}', xbmcgui.NOTIFICATION_ERROR, 3000)

def show_episodes(series_name, season_index, season_data):
    try:
        episodes = json.loads(season_data)
        
        for episode in episodes:
            if not isinstance(episode, dict) or 'name' not in episode:
                continue
            
            list_item = xbmcgui.ListItem(label=episode['name'])
            
            # Aggiungi informazioni aggiuntive (esempio: gruppo, staff_id, ecc.)
            info_text = f"Gruppo: {episode.get('group_title', 'N/A')}\n"
            info_text += f"Staff ID: {episode.get('staff_id', 'N/A')}"
            
            list_item.setInfo('video', {
                'title': episode['name'],
                'plot': info_text,
                'mediatype': 'episode'
            })
            
            # Aggiungi proprietà per il play
            list_item.setProperty('IsPlayable', 'true')
            
            # Se l'episodio ha un logo, imposta le immagini
            if 'logo' in episode and episode['logo']:
                list_item.setArt({
                    'thumb': episode['logo'],
                    'fanart': episode['logo']
                })
            
            # In show_episodes
            url = build_url({
                'action': 'play_episode',
                'episode_data': json.dumps({
                    'name': episode['name'],
                    'mpd_url': episode.get('mpd_url', ''),  # Assicurati che questo campo esista
                    'license_key': episode.get('license_key', ''),  # Se necessario
                    'staff_id': episode.get('staff_id', ''),
                    'logo': episode.get('logo', '')
                })
            })
            
            # Aggiungi l'episodio alla lista
            xbmcplugin.addDirectoryItem(handle=addon_handle, url=url, listitem=list_item, isFolder=False)
        
        xbmcplugin.endOfDirectory(addon_handle)
        
    except json.JSONDecodeError as e:
        xbmc.log(f"JSON decode error in show_episodes: {str(e)}", xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', 'Errore nel decodificare i dati', xbmcgui.NOTIFICATION_ERROR, 3000)
    except Exception as e:
        xbmc.log(f"Error in show_episodes: {str(e)}", xbmc.LOGERROR)
        xbmcgui.Dialog().notification('Errore', f'Errore: {str(e)}', xbmcgui.NOTIFICATION_ERROR, 3000)

def get_season_info(series_id, season_number):
    """
    Ottiene le informazioni specifiche di una stagione da TMDB.
    """
    tmdb_api_key = "ded5b7afc33c6be7e0eaefce452217b1"
    try:
        r = requests.get(
            f"https://api.themoviedb.org/3/tv/{series_id}/season/{season_number}",
            params={
                'api_key': tmdb_api_key,
                'language': 'it-IT'
            }
        )
        if r.status_code == 200:
            return r.json()
    except Exception as e:
        xbmc.log(f"Errore nel recupero delle informazioni della stagione: {str(e)}", xbmc.LOGERROR)
    return None

# Router per gestire le azioni
def router(paramstring):
    params = dict(parse_qsl(paramstring[1:]))
    if params:
        if params['action'] == 'open_category':
            open_category(params['category'])
        elif params['action'] == 'search_movies':
            search_movies()
        elif params['action'] == 'loadLiveTV':
            loadLiveTV(params['subcategory'])
        elif params['action'] == 'initCategory':
            initCategory(params['subcategory'])
        elif params['action'] == 'load_next_list':
            # Ottieni tutte le URL per la sottocategoria
            json_urls = get_json_urls_for_category(params['subcategory'])
            current_index = int(params['current_index'])
            total_lists = int(params['total_lists'])
            if current_index < len(json_urls):
                load_movie_list(json_urls[current_index], current_index, total_lists, params['subcategory'])
        elif params['action'] == 'show_seasons':
            show_seasons(params['series_name'], params['series_data'])
        elif params['action'] == 'show_episodes':
            show_episodes(
                params['series_name'],
                int(params['season_index']),
                params['season_data']
            )
        elif params['action'] == 'play':
            if "mpd_url" in params:
                play_mpd_content(params['mpd_url'], addon_handle)
            elif "hls_url" in params:
                play_hls_content(params['hls_url'], addon_handle)
        elif params['action'] == 'play_episode':
            # Passa i dati dell'episodio alla funzione play_episode
            episode_data = json.loads(params['episode_data'])
            play_episode(episode_data)
    else:
        main_menu()

# Avvio dell'addon
if __name__ == '__main__':
    router(sys.argv[2])